"""file: dec.py -- corection td3, exercise about number decomposition

"""

import random

def dec2int(dec,i=0):
  """dec is a list of characters '0', ..., '9'.
  
  dec2int(dec) returns the integer whose decimal decomposition is dec.

  """
  if dec == []:
    return 0 # base case
  else:
    return int(dec[0])*10**(len(dec)-1)+dec2int(dec[1:],i+1)

# remark: it is inefficient to compute len(dec) at each step, and a
# power of 10, a quicker version can be obtained by first reversing
# the order of the list dec.

def dec2int2_aux(dec):
  """dec is a list of characters '0', ..., '9'.  

  dec2int2_aux(dec) returns the integer whose decimal decomposition is dec[::-1].

  """
  if dec == [] : return 0
  else:
    return int(dec[0])+10*dec2int2_aux(dec[1:])


def dec2int2(dec):
  """same as dec2int.

  """
  return dec2int2_aux(dec[::-1])

# none of the above are tail recursive; use an accumulator for that

def dec2int3(dec, acc=0):
  """dec is a list of characters '0', ..., '9'; acc is an integer.
  
  dec2int(dec,acc) returns the integer whose decimal decomposition is
  dec added with acc.

  """
  if dec == []: return acc
  else:
    return dec2int3(dec[1:],int(dec[0])*10**(len(dec)-1)+acc)

## move on to the inverse function: give decimal decomposition of a
## given integer

def int2dec_aux(n):
  """n is an integer > 0

  int2dec_aux(n) returns the reverse list of the decimal decomposition of
  n.

  """
  if n < 10: return [str(n)]
  else: return [str(n%10)] + int2dec_aux(n//10)

def int2dec(n):
  """wrap up to get the decimal decomposition of n in the usual sense

  """
  return int2dec_aux(n)[::-1]

# it is not terminal recursive, but we can make it so by adding an
# accumulator

def int2dec2(n,acc=[]):
  """n is an integer > 0

  int2dec2(n) returns the decimal decomposition of n.

  """
  if n < 10: return [str(n)] + acc
  else: return int2dec2(n//10,[str(n%10)] + acc)

# iterative version
def int2dec_it(n):
  dec = []
  while n>=10:
    dec = [str(n%10)] + dec
    n = n//10
  return [str(n)] + dec

# move on to the conversion from dec to hex; maybe the best way is to
# convert from dec to integer then from integer to hex.

# first define the hexadecimal alphabet as a tuple
hexalph = ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F')

def int2hex(n,acc=[]): #analog of int2dec2
  """n is an integer > 0

  int2hex(n) returns the hexadecimal decomposition of n.

  """
  if n < 16: return [hexalph[n]] + acc
  else: return int2hex(n//16,[hexalph[n%16]] + acc)

def dec2hex(n): return int2hex(dec2int(n))
  
## testing
# alph = [str(i) for i in range(10)]
# li = [[random.choice(alph) for j in range(6)] for i in range(20)]
# for x in li:
#   print x, "is decimal for", dec2int(x)
#   print x, "is decimal for", dec2int2(x)
#   print x, "is decimal for", dec2int3(x)
#   print
# for x in [random.randrange(10,100) for i in range(20)]:
#   print x, "in decimal:", int2dec(x)
#   print x, "in decimal:", int2dec2(x)
#   print x, "in decimal:", int2dec_it(x)
#   print x, "in hex:", int2hex(x)
#   print
decalph = [str(i) for i in range(10)]
li = [[random.choice(decalph) for j in range(6)] for i in range(20)]
for x in li:
  print x, "in dec is", dec2hex(x), "in hex"
